#!/usr/bin/python

import os, sys, string, re, tempfile
import PIL.Image, PIL.ImageDraw

# receive commands
class ContentState:
    def __init__( self ):
        self.state = { 'fill': ( 0, 0, 0 ), 'width': 1 }
        pass
    def get_width( self ):
        return self.state[ 'width' ]

    def get_fill( self ):
        return self.state[ 'fill' ]

    def get_attribute( self, key ):
        return self.state[ key ]
    
    def set_attribute( self, key, value ):
        self.state[ key ] = value
        pass
    pass

class Command:
    def __init__( self, state ):
        self.state = state
        pass

    def get_attribute( self, key ):
        return self.state.get_attribute( key )
    
    def set_attribute( self, key, value ):
        self.state.set_attribute( key, value )
        pass

    def get_state( self ):
        return self.state

    def execute( self, draw ):
        raise TypeError( "not implemented in " + repr( self.__class__ ) )
        pass
    pass

class LineCommand ( Command ):
    def __init__( self, points, state ):
        Command.__init__( self, state )
        self.points = points
        pass

    def execute( self, draw ):
        fill = self.get_attribute( 'fill' )
        width = self.get_attribute( 'width' )
        n = len( self.points )
        for i in range( 1, n ):
            draw.line( self.points[ i - 1 ] + self.points[ i ], width=width, fill=fill )
            pass
        pass

    def draw_offset( self, draw, dx, dy ):
        fill = self.get_attribute( 'fill' )
        width = self.get_attribute( 'width' )
        n = len( self.points )
        for i in range( 1, n ):
            x0 = self.points[ i - 1 ][ 0 ] + dx
            y0 = self.points[ i - 1 ][ 1 ] + dy
            x1 = self.points[ i ][ 0 ] + dx
            y1 = self.points[ i ][ 1 ] + dy
            draw.line( (x0, y0, x1, y1 ), width=width, fill=fill )
            pass
        pass

    def generate_variation( self, vx, vy ):
        import random
        points = []
        for p in self.points:
            x = p[ 0 ] + int( vx * ( random.random() - .5 ) )
            y = p[ 1 ] + int( vy * ( random.random() - .5 ) )
            points.append( ( x, y, ) )
            pass
        return LineCommand( points, self.get_state() )
    pass

class ColorCommand( Command ):
    def __init__( self, color, state ):
        Command.__init__( self, state )
        self.color = tuple( color )
        pass

    def execute( self, draw ):
        self.set_attribute( 'fill', self.color )
        pass
    pass

class WidthCommand( Command ):
    def __init__( self, width, state ):
        Command.__init__( self, state )
        self.width = width
        pass

    def execute( self, draw ):
        self.set_attribute( 'width', self.width )
        pass
    pass
    


size = ( 250, 250 )
vibration = ( 10, 10 )
filename_output = tempfile.mktemp( ".gif" );
frames = 4
delay = 5
shadow = None
shadow_pos = ( 2, 5 )
transparency = None
background = ( 255, 255, 255 )
args = sys.argv[ 0: ]
args.reverse()
args.pop()
flag_serial = False
num_extend = 4

while True:
    try:
        arg = args.pop()
    except:
        break
    if arg == '-s':
        size_str = args.pop().split( 'x' )
        size = ( string.atoi( size_str[ 0 ] ), string.atoi( size_str[ 1 ] ) )
    elif arg == '-v':
        size_str = args.pop().split( 'x' )
        vibration = ( string.atoi( size_str[ 0 ] ), string.atoi( size_str[ 1 ] ) )
        pass
    elif arg == '-f':
        frames = string.atoi( args.pop() )
    elif arg == '-o':
        filename_output = args.pop()
        pass
    elif arg == '-d':
        delay = string.atoi( args.pop() )
        pass
    elif arg == '-shadow':
        col_str = args.pop().split( ',' )
        shadow = tuple( [ string.atoi( x ) for x in col_str[ 0 : 3 ] ] )
    elif arg == '-spos':
        col_str = args.pop().split( ',' )
        shadow_pos = tuple( [ string.atoi( x ) for x in col_str[ 0 : 3 ] ] )
    elif arg == '-tr':
        transparency = 0
        pass
    elif arg == '-bg':
        col_str = args.pop().split( ',' )
        background = tuple( [ string.atoi( x ) for x in col_str[ 0 : 3 ] ] )
        pass
    elif arg == '-serial':
        flag_serial = True
        pass
    pass

if flag_serial: num_expand = 4


line = sys.stdin.readline()
procedure = []
framenum_line = [ 0, ]
sharedstate = ContentState()
while line != "" and line.strip() != 'quit' and line.strip() != 'exit':
    command = line.strip().split( ',' )
    if command[ 0 ] == 'l': # line
        n = 1
        points = []
        while n + 2 <= len( command ):
            x = string.atoi( command[ n ] )
            y = string.atoi( command[ n + 1 ] )
            points.append( ( x, y, ) )
            n += 2
            pass
        framenum_line.append( len( procedure ) )
        procedure.append( LineCommand( points, sharedstate ) )        
        #lines.append( points )
        pass
    elif command[ 0 ] == 'w': # width
        procedure.append( WidthCommand( string.atoi( command[ 1 ] ), sharedstate ) )
    elif command[ 0 ] == 'c':
        procedure.append( ColorCommand( [ string.atoi( x ) for x in command[ 1: 4 ] ], sharedstate ) )
        pass
    line = sys.stdin.readline()
    pass

if not flag_serial:
    framenum_line = [ len( procedure ) - 1, ]
else:
    num_end = framenum_line[ -1 ]
    for i in range( num_extend ): framenum_line.append( num_end )
    pass


# initialize

filenames_tmp = []
num_start = 0
num_end = 0
num_frames = 0
for n in framenum_line:
    num_end = n + 1

    for i in range( 0, frames ):
        num = ("00%d" % ( i + num_frames ) )[ -2: ]
        filename_tmp = tempfile.mktemp( '%s.gif' % num,  'drawing' )
        filenames_tmp.append( filename_tmp )
        pass

    if not shadow:
        for i in range( 0, frames ):
            im = PIL.Image.new( 'RGB', size )
            draw = PIL.ImageDraw.Draw( im )
            draw.rectangle( ( 0, 0 ) + size, fill=background )
            for j in range( 0, num_end ):
                command = procedure[ j ]
                #for command in procedure:
                if isinstance( command, LineCommand ):
                    command = command.generate_variation(  5, 5 )
                    pass
                command.execute( draw )
                pass
            im.save( filenames_tmp[ i + num_frames ] )
            pass
        pass
    else:
        for i in range( 0, frames ):
            im = PIL.Image.new( 'RGB', size )
            draw = PIL.ImageDraw.Draw( im )
            draw.rectangle( ( 0, 0 ) + size, fill=background )
            #shadow
            procedure[ 0 ].set_attribute( 'fill', shadow )
            command_face = []
            for j in range( 0, num_end ):
                command = procedure[ j ]
                #for command in procedure:
                if isinstance( command, LineCommand ):
                    command = command.generate_variation( vibration[ 0 ], vibration[ 1 ] )
                    command.draw_offset( draw, shadow_pos[ 0 ], shadow_pos[ 1 ] )
                    pass
                elif isinstance( command, ColorCommand ) == False:
                    command.execute( draw )
                    pass

                command_face.append( command )
                pass
            #face
            for command in command_face:
                command.execute( draw )
                pass

            im.save( filenames_tmp[ i + num_frames ], transparency=225 )
            #print filenames_tmp
            pass
        pass
    num_start = num_end
    num_frames += frames

# collect
command = "./whirlgif -o %s -loop 0  " % ( filename_output )
for fn in filenames_tmp:
    command += '%s ' % ( fn )
    #command += '-time %d %s ' % ( delay, fn )
    pass
os.system( command + "2> /dev/null" )
#print command
for fn in filenames_tmp:
    os.unlink( fn )
    pass

if os.path.exists( filename_output ):
    print filename_output
    #print "file://" + os.path.abspath( filename_output )
else:
    print "NO FILE"
